@@ -14,22 +14,12 @@
 #define IS_PY3K
 #endif
 
-#ifdef STDC_HEADERS
-#include <stddef.h>
-#else  /* !STDC_HEADERS */
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>      /* For size_t */
-#endif /* HAVE_SYS_TYPES_H */
-#endif /* !STDC_HEADERS */
-
-
-typedef long long int idx_t;
 
 typedef struct {
     PyObject_VAR_HEAD
-    char *ob_item;
+    char *ob_item;              /* buffer */
     Py_ssize_t allocated;       /* how many bytes allocated */
-    idx_t nbits;                /* length of bitarray, i.e. elements */
+    Py_ssize_t nbits;           /* length of bitarray, i.e. elements */
     int endian;                 /* bit endianness of bitarray */
     int ob_exports;             /* how many buffer exports */
     PyObject *weakreflist;      /* list of weak references */
@@ -38,7 +28,7 @@ typedef struct {
 #define ENDIAN_LITTLE  0
 #define ENDIAN_BIG     1
 
-#define BITS(bytes)  ((idx_t) (bytes) << 3)
+#define BITS(bytes)  ((bytes) << 3)
 
 #define BYTES(bits)  (((bits) == 0) ? 0 : (((bits) - 1) / 8 + 1))
 
@@ -48,7 +38,7 @@ typedef struct {
 /* ------------ low level access to bits in bitarrayobject ------------- */
 
 #ifndef NDEBUG
-static int GETBIT(bitarrayobject *self, idx_t i) {
+static int GETBIT(bitarrayobject *self, Py_ssize_t i) {
     assert(0 <= i && i < self->nbits);
     return ((self)->ob_item[(i) / 8] & BITMASK((self)->endian, i) ? 1 : 0);
 }
@@ -58,27 +48,17 @@ static int GETBIT(bitarrayobject *self, idx_t i) {
 #endif
 
 static void
-setbit(bitarrayobject *self, idx_t i, int bit)
+setunused(bitarrayobject *self)
 {
-    char *cp, mask;
+    const char mask[16] = {
+        0x00, 0x01, 0x03, 0x07, 0x0f, 0x1f, 0x3f, 0x7f, /* little endian */
+        0x00, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0xfc, 0xfe, /* big endian */
+    };
 
-    assert(0 <= i && i < BITS(Py_SIZE(self)));
-    mask = BITMASK(self->endian, i);
-    cp = self->ob_item + i / 8;
-    if (bit)
-        *cp |= mask;
-    else
-        *cp &= ~mask;
-}
-
-static void
-setunused(bitarrayobject *a)
-{
-    const idx_t n = BITS(Py_SIZE(a));
-    idx_t i;
-
-    for (i = a->nbits; i < n; i++)
-        setbit(a, i, 0);
+    if (self->nbits % 8 == 0)
+        return;
+    self->ob_item[Py_SIZE(self) - 1] &=
+        mask[self->nbits % 8 + (self->endian == ENDIAN_LITTLE ? 0 : 8)];
 }
 
 static unsigned char bitcount_lookup[256] = {
@@ -94,30 +74,37 @@ static unsigned char bitcount_lookup[256] = {
 
 /*********** end of code basically copied from _bitarray.c *************/
 
-/* set using the Python module function _set_babt() */
-static PyObject *bitarray_basetype = NULL;
+/* set using the Python module function _set_bato() */
+static PyObject *bitarray_type_obj = NULL;
 
-/* Return 1 if obj is a bitarray, 0 otherwise.
-   Note that this is implemented differently in _bitarray.c */
+/* Return 0 if obj is bitarray.  If not, returns -1 and sets an exception. */
 static int
-bitarray_Check(PyObject *obj)
+ensure_bitarray(PyObject *obj)
 {
-    if (bitarray_basetype == NULL) {
-        fprintf(stderr, "FATAL: bitarray_basetype missing\n");
-        exit(1);
+    int t;
+
+    if (bitarray_type_obj == NULL)
+        Py_FatalError("bitarray_type_obj missing");
+    t = PyObject_IsInstance(obj, bitarray_type_obj);
+    if (t < 0)
+        return -1;
+    if (t == 0) {
+        PyErr_SetString(PyExc_TypeError, "bitarray expected");
+        return -1;
     }
-    return PyObject_IsInstance(obj, bitarray_basetype);
+    return 0;
 }
 
 /************ start of actual functionality in this module *************/
 
 /* return the smallest index i for which a.count(1, 0, i) == n, or when
    n exceeds the total count return -1  */
-static idx_t
-count_to_n(bitarrayobject *a, idx_t n)
+static Py_ssize_t
+count_to_n(bitarrayobject *a, Py_ssize_t n)
 {
-    idx_t i = 0, j = 0, m;  /* i is the index, j the total count up to i */
-    Py_ssize_t block_start, block_stop, k;
+    Py_ssize_t i = 0;        /* index */
+    Py_ssize_t j = 0;        /* total count up to index */
+    Py_ssize_t block_start, block_stop, k, m;
     unsigned char c;
 
     if (n == 0)
@@ -128,7 +115,7 @@ count_to_n(bitarrayobject *a, idx_t n)
     while (i + BLOCK_BITS < a->nbits) {
         m = 0;
         assert(i % 8 == 0);
-        block_start = (Py_ssize_t) (i / 8);
+        block_start = i / 8;
         block_stop = block_start + (BLOCK_BITS / 8);
         for (k = block_start; k < block_stop; k++) {
             assert(k < Py_SIZE(a));
@@ -143,7 +130,7 @@ count_to_n(bitarrayobject *a, idx_t n)
 #undef BLOCK_SIZE
 
     while (i + 8 < a->nbits) {
-        k = (Py_ssize_t) (i / 8);
+        k = i / 8;
         assert(k < Py_SIZE(a));
         c = a->ob_item[k];
         m = bitcount_lookup[c];
@@ -164,11 +151,10 @@ count_to_n(bitarrayobject *a, idx_t n)
 }
 
 /* return index of last occurrence of vi, -1 when x is not in found. */
-static idx_t
+static Py_ssize_t
 find_last(bitarrayobject *a, int vi)
 {
-    Py_ssize_t j;
-    idx_t i;
+    Py_ssize_t i, j;
     char c;
 
     if (a->nbits == 0)
@@ -223,15 +209,14 @@ static PyObject *
 count_n(PyObject *module, PyObject *args)
 {
     PyObject *a;
-    idx_t n, i;
+    Py_ssize_t n, i;
 
-    if (!PyArg_ParseTuple(args, "OL:count_n", &a, &n))
+    if (!PyArg_ParseTuple(args, "On:count_n", &a, &n))
         return NULL;
 
-    if (!bitarray_Check(a)) {
-        PyErr_SetString(PyExc_TypeError, "bitarray expected");
+    if (ensure_bitarray(a) < 0)
         return NULL;
-    }
+
     if (n < 0) {
         PyErr_SetString(PyExc_ValueError, "non-negative integer expected");
         return NULL;
@@ -254,23 +239,22 @@ PyDoc_STRVAR(count_n_doc,
 "count_n(a, n, /) -> int\n\
 \n\
 Find the smallest index `i` for which `a[:i].count() == n`.\n\
-Raises `ValueError`, when n exceeds the `a.count()`.");
+Raises `ValueError`, when n exceeds total count (`a.count()`).");
 
 
 static PyObject *
 r_index(PyObject *module, PyObject *args)
 {
-    PyObject *a, *x = Py_True;
-    idx_t i;
+    PyObject *x = Py_True, *a;
+    Py_ssize_t i;
     int vi;
 
     if (!PyArg_ParseTuple(args, "O|O:rindex", &a, &x))
         return NULL;
 
-    if (!bitarray_Check(a)) {
-        PyErr_SetString(PyExc_TypeError, "bitarray expected");
+    if (ensure_bitarray(a) < 0)
         return NULL;
-    }
+
     vi = PyObject_IsTrue(x);
     if (vi < 0)
         return NULL;
@@ -300,57 +284,51 @@ enum kernel_type {
 static PyObject *
 two_bitarray_func(PyObject *args, enum kernel_type kern, char *format)
 {
+    Py_ssize_t res = 0, nbytes, i;
     PyObject *a, *b;
-    Py_ssize_t n, i;
-    idx_t res = 0;
     unsigned char c;
 
     if (!PyArg_ParseTuple(args, format, &a, &b))
         return NULL;
-    if (!(bitarray_Check(a) && bitarray_Check(b))) {
-        PyErr_SetString(PyExc_TypeError, "bitarray expected");
+    if (ensure_bitarray(a) < 0 || ensure_bitarray(b) < 0)
         return NULL;
-    }
+
 #define aa  ((bitarrayobject *) a)
 #define bb  ((bitarrayobject *) b)
-    if (aa->nbits != bb->nbits) {
+    if (aa->nbits != bb->nbits || aa->endian != bb->endian) {
         PyErr_SetString(PyExc_ValueError,
-                        "bitarrays of equal length expected");
-        return NULL;
-    }
-    if (aa->endian != bb->endian) {
-        PyErr_SetString(PyExc_ValueError,
-                        "bitarrays of equal endianness expected");
+                        "bitarrays of equal length and endianness expected");
         return NULL;
     }
     setunused(aa);
     setunused(bb);
     assert(Py_SIZE(a) == Py_SIZE(b));
-    n = Py_SIZE(a);
+    nbytes = Py_SIZE(a);
 
     switch (kern) {
     case KERN_cand:
-        for (i = 0; i < n; i++) {
+        for (i = 0; i < nbytes; i++) {
             c = aa->ob_item[i] & bb->ob_item[i];
             res += bitcount_lookup[c];
         }
         break;
     case KERN_cor:
-        for (i = 0; i < n; i++) {
+        for (i = 0; i < nbytes; i++) {
             c = aa->ob_item[i] | bb->ob_item[i];
             res += bitcount_lookup[c];
         }
         break;
     case KERN_cxor:
-        for (i = 0; i < n; i++) {
+        for (i = 0; i < nbytes; i++) {
             c = aa->ob_item[i] ^ bb->ob_item[i];
             res += bitcount_lookup[c];
         }
         break;
     case KERN_subset:
-        for (i = 0; i < n; i++)
+        for (i = 0; i < nbytes; i++) {
             if ((aa->ob_item[i] & bb->ob_item[i]) != aa->ob_item[i])
                 Py_RETURN_FALSE;
+        }
         Py_RETURN_TRUE;
     default:  /* should never happen */
         return NULL;
@@ -392,11 +370,11 @@ efficient since we can stop as soon as one mismatch is found, and no\n\
 intermediate bitarray object gets created.");
 
 
-/* set bitarray_basetype (babt) */
+/* set bitarray_type_obj (bato) */
 static PyObject *
-set_babt(PyObject *module, PyObject *obj)
+set_bato(PyObject *module, PyObject *obj)
 {
-    bitarray_basetype = obj;
+    bitarray_type_obj = obj;
     Py_RETURN_NONE;
 }
 
@@ -407,7 +385,7 @@ static PyMethodDef module_functions[] = {
     {"count_or",  (PyCFunction) count_or,  METH_VARARGS, count_or_doc},
     {"count_xor", (PyCFunction) count_xor, METH_VARARGS, count_xor_doc},
     {"subset",    (PyCFunction) subset,    METH_VARARGS, subset_doc},
-    {"_set_babt", (PyCFunction) set_babt,  METH_O,       ""},
+    {"_set_bato", (PyCFunction) set_bato,  METH_O,       },
     {NULL,        NULL}  /* sentinel */
 };
 
