@@ -1,6 +1,8 @@
 """
 Tests for bitarray.util module
 """
+from __future__ import absolute_import
+
 import os
 import sys
 import unittest
@@ -218,9 +220,13 @@ class TestsHelpers(unittest.TestCase, Util):
         self.assertEqual(count_n(a, 0), 0)
         self.assertEqual(count_n(a, 20), 23)
         self.assertEqual(count_n(a, 37), 45)
-        self.assertRaises(ValueError, count_n, a, -1) # n < 0
-        self.assertRaises(ValueError, count_n, a, 38) # n > a.count()
-        self.assertRaises(ValueError, count_n, a, 49) # n > len(a)
+        self.assertRaisesMessage(ValueError,
+                                 "non-negative integer expected",
+                                 count_n, a, -1) # n < 0
+        self.assertRaisesMessage(ValueError, "n larger than bitarray size",
+                                 count_n, a, 49) # n > len(a)
+        self.assertRaisesMessage(ValueError, "n exceeds total count",
+                                 count_n, a, 38) # n > a.count()
         self.assertRaises(TypeError, count_n, a, "7")
         for n in range(0, 37):
             i = count_n(a, n)
